#pragma once
#include <string>
#include <map>
#include <vector>
#include <memory>
#include <functional>
#include "raylib.h"
#include "ServiceLocator.h"
#include "Scene.h"

using namespace std;

namespace LE
{
	class Engine;
	class IEngineService : public IService
	{
	public:
		virtual void SendLateUpdateCommand(function<void(LE::Engine&)> InNewCommand) = 0;

		virtual void SetClearColor(Color InColor) = 0;
		virtual const Color& GetClearColor() const = 0;
		virtual void SetDebugMode(bool InDebugMode) = 0;
		virtual bool GetDebugMode() const = 0;
		virtual bool IsSceneRegistered(const string& InNewSceneName) const = 0;
	};

	//Fonction globale pour envoyer des commandes a Engine via le Service Locator
	//C'est juste un raccourcis au lieu de directement passer par le Service Locator
	extern void SendCommand(function<void(LE::Engine&)> InNewCommand);

	class Engine : public ServiceBase<IEngineService>
	{
	public:
		//Log INFO:  la fin du constructeur, signaler que Engine est construit.
		//Log ERROR: Resolution ngative, TargetFPS en dehors de [5,100].
		Engine(int InResolutionX = 800, int InResolutionY = 480, const string& InTitle = "Sans titre", int InTargetFPS = 60);
		virtual ~Engine();

		//Appeler cette mthode pour ouvrir la fentre et dmarrer la boucle while
		//Log INFO: Signaler que le jeu a dmarr avec la scne InStartSceneName
		//Log ERROR: Scne de dmarrage n'existe pas.
		//Log ERROR: Aucune scne enregistre.
		void StartGame(const string& InStartSceneName);

		//Couleur de fond de l'application
		void SetClearColor(Color InColor);
		const Color& GetClearColor() const;

		//Si  vrai, va afficher le FPS en haut  droite et le message suivant: "Press P to toggle debug"
		//Codez aussi que si l'usager appuie sur la touche p, le mode debug change de vrai  faux ou l'inverse
		void SetDebugMode(bool InDebugMode);
		bool GetDebugMode() const;

		//*=*=*=*=*=*=*=*AVANC=*=*=*=*=*=*=*=*=//
		//Permet d'excuter une mthode entre 2 frames
		virtual void SendLateUpdateCommand(function<void(LE::Engine&)> InNewCommand);
		//*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=//

		//Scene register/unregister
		//Log INFO: Scne "InNewSceneName" est enregistre
		//Log ERROR: Scne "InNewSceneName" existe dj.
		//Log ERROR: Nom fourni est vide.
		void RegisterScene(shared_ptr<LE::Scene> NewScene);
		void UnregisterScene(const string& InSceneName);
		shared_ptr<LE::Scene> FindScene(const string& InSceneName);
		weak_ptr<LE::Scene> GetCurrentActiveScene();

		//Si appel, on doit ferm l'application en quittant le "while" de manire propre.
		void RequestClose();

		//Change la scne active
		//Log INFO: Scne "InNewSceneName" maintenant active.
		//Log WARNING: Scne "InNewSceneName" est dj active.
		//Log ERROR: Scne "InNewSceneName" n'existe pas.
		void SwitchScene(const string&  InNewSceneName);
		//Vrifie si la scne est enregistre dans Engine
		bool IsSceneRegistered(const string& InNewSceneName) const;
	};
}

