#pragma once
#include <string>
#include <memory>
#include <vector>
#include "BaseObject.h"
#include "GameObject.h"

namespace LE
{
	class Scene : public IBaseObject
	{
		public:
			//Log ERROR: Nom de scne vide.
			Scene(const std::string& InName);
			virtual ~Scene();

			//Retourne le nom de la scne.
			const std::string& GetName() const;

			//Ajoute un GameObject2D de la collection
			//Si la scne est active, on appel Load sur l'objet ajout
			//Log WARNING: Objet nomm X dj dans la scne, l'objet n'est pas ajout.
			void AddGameObject2D(std::shared_ptr<GameObject2D> InNewGameObject);

			//Enlve un GameObject2D
			//Si la scne est active, on appel Unload sur l'objet ajout
			//Log WARNING: Objet nomm X n'est pas dans dans la scne, l'objet n'est pas enlev.
			void RemoveGameObject2D(std::shared_ptr<GameObject2D> InRemovedGameObject);
			bool HasGameObject2D(std::shared_ptr<GameObject2D> InGameObject);

			//Appel Load sur les GameObject2D
			//Log WARNING: Scne dj load, la scne ne reloadera pas.
			void Load();
			//Appel Unload sur les GameObject2D
			//Log WARNING: Scne n'est pas loade, la scne ne dloadera pas.
			virtual void Unload();

			//Les mthodes suivantes sont appeles dans l'ordre dans le while  chaque image affiche.
			//Effectue les test de collisions, puis appelle PreUpdate sur les GameObject2D
			virtual void PreUpdate();
			//Appel Draw3D sur les GameObject2D
			virtual void Draw3D();
			//Appel Draw2D sur les GameObject2D
			virtual void Draw2D();
			//Compare tous les objets entre eux pour voir s'ils collisionnent et appel le fonction de callback au besoin
			//Filtre selon les masque de collision
			virtual void CheckAndNotifyCollision();
			//Appel LateUpdate sur les GameObject2D
			virtual void LateUpdate();
	};
}