﻿#include <iostream>
#include <memory>
//Les "" sont toujours relatifs au dossier racine du projet, contrairement aux dossiers <>
#include "raylib.h"
#include "LoopEngine/Engine.h"
#include "LoopEngine/Scene.h"
#include "LoopEngine/ServiceLocator.h"
#include "LoopEngine/Logger.h"

using namespace LE;

//Exemple de fonction de test
void TestLogger()
{
    std::shared_ptr<LE::Logger> logger = std::make_shared<LE::Logger>();
    service_locator.registerService(logger);
    logger->SetLoggingLevel(TLevel::eINFO);
    logger->Log("Warning coucou, du pointeur de logger!",TLevel::eWARNING);
    logger->Log("Info coucou, du pointeur de logger!",TLevel::eINFO);

    //Accéder au service locator 
    service_locator.getService<ILoggerService>()->Log("Warning avec acces à partir du service locator",TLevel::eWARNING);
    service_locator.getService<ILoggerService>()->SetAbortLevel(eNONE);
    service_locator.getService<ILoggerService>()->Log("Error avec acces à partir du service locator",TLevel::eERROR);

    //Test pour construire la macro
    service_locator.getService<ILoggerService>()->Log(std::string(__FILENAME__) + ":" + to_string(__LINE__) + "Test pour fichier et ligne", TLevel::eWARNING);
    //Test une fois la macro créée
    LOG("La macro fonctionne!", TLevel::eINFO);
}

void TestLoopEngine()
{
    //Enregistrement des services
    std::shared_ptr<LE::Logger> logger = std::make_shared<LE::Logger>();
    //Log INFO: Nouveau service enregistré.
    service_locator.registerService(logger);
    std::shared_ptr<LE::Engine> engine = std::make_shared<LE::Engine>();
    //Log INFO: Nouveau service enregistré.
    service_locator.registerService(engine);


    //Configuration et démarrage du moteur
    if (engine != nullptr)
    {
        engine->SetClearColor(PINK);
        //Scène vide
        engine->RegisterScene("Menu", std::make_unique<LE::Scene>("Menu"));
        engine->StartGame("Menu");
    }
}

int main(void)
{
    TestLogger();
    //TestLoopEngine();
    return 0;
}